from django.db import models
from cms.models.pagemodel import Page

from tagging.fields import TagField

class PageTagging(models.Model):
    page = models.OneToOneField(Page)
    page_tags = TagField()



import random
import string
from django.forms.widgets import TextInput
from django.utils.safestring import mark_safe
from django.contrib import admin
from django.contrib.admin.sites import NotRegistered
from django.core.urlresolvers import reverse


from tagging.fields import TagField
from cms.models.pagemodel import Page

from models import PageTagging


def _get_registered_modeladmin(model):
    """ This is a huge hack to get the registered modeladmin for the model.
        We need this functionality in case someone else already registered
        a different modeladmin for this model. """
    return type(admin.site._registry[model])


class AutocompleteWidget(TextInput):
    def render(self, *args, **kwargs):
        id = (kwargs
                .get('attrs', {})
                .get('id', random.sample(string.letters, 15)))
        html = super(AutocompleteWidget, self).render(*args, **kwargs)
        url = reverse('pagetags.views.list_tags')
        js = '<script type="text/javascript">autocomplete("%s", "%s")</script>'
        js %= (id, url)
        return mark_safe(html + js)


class PageTaggingAdmin(admin.TabularInline):
    model = PageTagging
    fields = ('page_tags',)
    formfield_overrides = {
        TagField: {
            'widget': AutocompleteWidget(attrs={'size':'50'})
        }
    }



RegisteredPageAdmin = _get_registered_modeladmin(Page)
RegisteredPageAdmin.inlines.append(PageTaggingAdmin)
RegisteredPageAdmin.change_form_template = 'pagetags/change_form.html'

try:
    admin.site.unregister(Page)
except NotRegistered:
    pass
admin.site.register(Page, RegisteredPageAdmin)



import re

from django import template
from django.conf import settings
from django.db.models import Q
from cms.models import Page

from tagging.models import TaggedItem
from tagging.utils import parse_tag_input
from pagetags.models import PageTagging


register = template.Library()


class StaticSlug(unicode):
    def bind(self, context):
        pass


class DynamicSlug():
    def __init__(self, varname):
        self.varname = varname
        self._value = u''

    def bind(self, context):
        self._value = context[self.varname]

    def __unicode__(self):
        return self._value


class StaticTags(list):
    def __init__(self, tags):
        super(StaticTags, self).__init__(parse_tag_input(tags))

    def bind(self, context):
        pass


class DynamicTags(list):
    def __init__(self, varname):
        self.varname = varname
        super(DynamicTags, self).__init__()

    def bind(self, context):
        self[:] = parse_tag_input(context.get(self.varname, ''))


class TaggedPagesNode(template.Node):
    def __init__(self, parsed_tags, ordering, limit, var_name):
        self.parsed_tags = parsed_tags
        self.ordering = ordering or 'chronological'
        self.limit = limit
        self.var_name = var_name

    def render(self, context):
        self.parsed_tags.bind(context)
        pagetaggings_from_site = PageTagging.objects.filter(
            page__site=settings.SITE_ID
        ).select_related()

        if self.ordering == 'alphabetical':
            pagetaggings_from_site = pagetaggings_from_site.order_by(
                'page__title_set__title'
            )

        if self.ordering == 'chronological':
            pagetaggings_from_site = pagetaggings_from_site.order_by(
                '-page__publication_date'
            )

        page_taggings = TaggedItem.objects.get_by_model(
            pagetaggings_from_site, self.parsed_tags
        )[:self.limit]

        context[self.var_name] = [
            page_tagging.page for page_tagging in page_taggings
        ]
        return ''


class SimilarPagesNode(template.Node):
    def __init__(self, parsed_slug, limit, var_name):
        self.parsed_slug = parsed_slug
        self.limit = limit
        self.var_name = var_name

    def render(self, context):
        self.parsed_slug.bind(context)
        try:
            page = Page.objects.get(
                Q(site=settings.SITE_ID),
                Q(title_set__slug=self.parsed_slug)
            )
        except Page.DoesNotExist:
            return ''

        page_taggings = TaggedItem.objects.get_related(
            page.pagetagging,
            PageTagging.objects.select_related().filter(
                page__site=settings.SITE_ID
            )
        )[:self.limit]
        context[self.var_name] = [
            page_tagging.page for page_tagging in page_taggings
        ]
        return ''


def _extract_tag_content(token):
    try:
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        raise template.TemplateSyntaxError(
            "%r tag requires additional arguments" % token.contents.split()[0]
        )
    return tag_name, arg


def parsed_tags(quoted_or_var):
    if (quoted_or_var[0] == quoted_or_var[-1]
    and quoted_or_var[0] in ('"', "'")):
        return StaticTags(tags=quoted_or_var[1:-1])
    return DynamicTags(varname=quoted_or_var)


def parse_slug(quoted_or_var):
    if (quoted_or_var[0] == quoted_or_var[-1]
    and quoted_or_var[0] in ('"', "'")):
        return StaticSlug(quoted_or_var[1:-1])
    return DynamicSlug(varname=quoted_or_var)


@register.tag
def pages_with_tags(parser, token):
    parsed_tags, ordering, limit, var_name = _parse_pages_with_tags(token)
    return TaggedPagesNode(parsed_tags, ordering, limit, var_name)


def _parse_pages_with_tags(token):
    tag_name, arg = _extract_tag_content(token)

    m = re.search(r"""
        (.*?)                                   # taglist or varname

        (\s+ order                              # optional order clause
            \s+(alphabetical|chronological)     # order combinations
        )*

        (\s+ limit \s+ (\d+))*                  # optional limit clause

        \s+ as \s+ (\w+) \s*                    # collector variable
    """, arg, flags=re.VERBOSE)

    if not m:
        raise template.TemplateSyntaxError(
            "%r tag had invalid arguments" % tag_name
        )
    tags, _, ordering, _, limit, var_name = m.groups()

    return parsed_tags(tags), ordering, limit and int(limit), var_name


@register.tag
def pages_similar_with(parser, token):
    parsed_slug, limit, var_name = _parse_pages_similar_with(token)
    return SimilarPagesNode(parsed_slug, limit, var_name)


def _parse_pages_similar_with(token):
    tag_name, arg = _extract_tag_content(token)

    m = re.search(r"""
        (.*?)                                   # slug or varname

        (\s+ limit \s+ (\d+))*                  # optional limit clause

        \s+ as \s+ (\w+) \s*                    # collector variable
    """, arg, flags=re.VERBOSE)
    if not m:
        raise template.TemplateSyntaxError(
            "%r tag had invalid arguments" % tag_name
        )
    page_slug, _, limit, var_name = m.groups()

    return parse_slug(page_slug), limit and int(limit), var_name



from setuptools import setup, find_packages

setup(
    name='django-cms-pagetags',
    version='0.1.4',
    description=('Tag fileds for CMS pages.'),
    keywords='django-cms django cms tags',
    author='Calin Furau',
    author_email='calin.furau@gmail.com',
    url='https://github.com/pbs/django-cms-pagetags',
    packages=find_packages(),
    include_package_data=True,
    install_requires = ['django-tagging==0.3.1'],

)



# Create your views here.

from django.http import HttpResponse
from tagging.models import Tag
import json

def list_tags(request):
    q = request.GET.get('q')
    tags = []
    if q:
        tags = (Tag.objects
                .filter(name__istartswith=q)
                .values_list('name', flat=True))
    response = [dict(id=tag, label=tag, value=tag) for tag in tags]
    return HttpResponse(json.dumps(response), mimetype='application/json')
